#!/usr/bin/env python

from distutils.core import setup
from Cython.Build import cythonize
import sys

extensions=cythonize("cysignals_example.pyx")

# Run Distutils
setup(
    name="cysignals_example",
    version='1.0',
    ext_modules=extensions,
    license='GNU Lesser General Public License, version 3 or later',
)



#!/usr/bin/env python
#
# Test interaction with gdb, i.e. the cysignals-CSI script
#

import unittest

import os
import sys
from subprocess import Popen, PIPE
from tempfile import mkdtemp
from shutil import rmtree

class TestGDB(unittest.TestCase):
    def setUp(self):
        # Store crash logs in a temporary directory
        self.crash_dir = mkdtemp()
        self.env = dict(os.environ)
        self.env["CYSIGNALS_CRASH_LOGS"] = self.crash_dir

    def tearDown(self):
        rmtree(self.crash_dir)

    def test_gdb(self):
        # Run a Python subprocess which we intentionally crash to inspect the
        # crash logfile.
        p = Popen([sys.executable], stdin=PIPE, env=self.env)
        with p.stdin as stdin:
            stdin.write(b"from cysignals.tests import *\n")
            stdin.write(b"unguarded_dereference_null_pointer()\n")

        ret = p.wait()
        self.assertLess(ret, 0)

        # Check crash log
        logs = [os.path.join(self.crash_dir, fn)
                for fn in os.listdir(self.crash_dir) if fn.endswith(".log")]
        self.assertEqual(len(logs), 1)
        log = open(logs[0]).read()

        self.assertIn(b"Stack backtrace", log)
        self.assertIn(b"Cython backtrace", log)
        self.assertIn(b"__pyx_pf_9cysignals_5tests_46unguarded_dereference_null_pointer()", log)
        self.assertIn(b"cdef void dereference_null_pointer() nogil:", log)


if __name__ == '__main__':
    unittest.main()



"""
Interface to the ``pselect()`` system call
==========================================

This module defines a class :class:`PSelecter` which can be used to
call the system call ``pselect()`` and which can also be used in a
``with`` statement to block given signals until
:meth:`PSelecter.pselect` is called.

Waiting for subprocesses
------------------------

One possible use is to wait with a **timeout** until **any child process**
exits, as opposed to ``os.wait()`` which doesn't have a timeout or
``multiprocessing.Process.join()`` which waits for one specific process.

Since ``SIGCHLD`` is ignored by default, we first need to install a
signal handler for ``SIGCHLD``. It doesn't matter what it does, as long
as the signal isn't ignored::

    >>> import signal
    >>> def dummy_handler(sig, frame):
    ...     pass
    >>> _ = signal.signal(signal.SIGCHLD, dummy_handler)

We wait for a child created using the ``subprocess`` module::

    >>> from cysignals.pselect import PSelecter
    >>> from subprocess import *
    >>> with PSelecter([signal.SIGCHLD]) as sel:
    ...     p = Popen(["sleep", "1"])
    ...     _ = sel.sleep()
    >>> p.poll()  # p should be finished
    0

Now using the ``multiprocessing`` module::

    >>> from cysignals.pselect import PSelecter
    >>> from multiprocessing import *
    >>> import time
    >>> with PSelecter([signal.SIGCHLD]) as sel:
    ...     p = Process(target=time.sleep, args=(1,))
    ...     p.start()
    ...     _ = sel.sleep()
    ...     p.is_alive()  # p should be finished
    False

"""

#*****************************************************************************
#       Copyright (C) 2013 Jeroen Demeyer <jdemeyer@cage.ugent.be>
#
#  cysignals is free software: you can redistribute it and/or modify it
#  under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  cysignals is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with cysignals.  If not, see <http://www.gnu.org/licenses/>.
#
#*****************************************************************************

cimport libc.errno
from posix.signal cimport *
from posix.select cimport *
from cpython.exc cimport PyErr_SetFromErrno


def interruptible_sleep(double seconds):
    """
    Sleep for ``seconds`` seconds or until a signal arrives. This
    behaves like ``time.sleep`` from Python versions <= 3.4
    (before :pep:`475`).

    EXAMPLES::

        >>> from cysignals.pselect import interruptible_sleep
        >>> interruptible_sleep(0.5)

    We set up an alarm handler doing nothing and check that the alarm
    interrupts the sleep::

        >>> import signal, time
        >>> def alarm_handler(sig, frame):
        ...     pass
        >>> _ = signal.signal(signal.SIGALRM, alarm_handler)
        >>> t0 = time.time()
        >>> _ = signal.alarm(1)
        >>> interruptible_sleep(2)
        >>> t = time.time() - t0
        >>> (1.0 <= t <= 1.9) or t
        True

    TESTS::

        >>> interruptible_sleep(0)
        >>> interruptible_sleep(-1)
        Traceback (most recent call last):
        ...
        ValueError: sleep length must be non-negative

    Reset the signal handlers::

        >>> from cysignals import init_cysignals
        >>> _ = init_cysignals()

    """
    if seconds < 0:
        raise ValueError("sleep length must be non-negative")

    cdef fd_set rfds, wfds, xfds
    FD_ZERO(&rfds)
    FD_ZERO(&wfds)
    FD_ZERO(&xfds)

    cdef timespec tv
    tv.tv_sec = <long>seconds
    tv.tv_nsec = <long>(1e9 * (seconds - <double>tv.tv_sec))

    cdef int ret = pselect(0, &rfds, &wfds, &xfds, &tv, NULL)
    if ret < 0:
        if libc.errno.errno != libc.errno.EINTR:
            PyErr_SetFromErrno(OSError)


cpdef int get_fileno(f) except -1:
    """
    Return the file descriptor of ``f``.

    INPUT:

    - ``f`` -- an object with a ``.fileno`` method or an integer,
      which is a file descriptor.

    OUTPUT: A C ``long`` representing the file descriptor.

    EXAMPLES::

        >>> from os import devnull
        >>> from cysignals.pselect import get_fileno
        >>> get_fileno(open(devnull)) > 2
        True
        >>> get_fileno(42)
        42
        >>> get_fileno(None)
        Traceback (most recent call last):
        ...
        TypeError: an integer is required
        >>> get_fileno(-1)
        Traceback (most recent call last):
        ...
        ValueError: Invalid file descriptor
        >>> get_fileno(2**30)
        Traceback (most recent call last):
        ...
        ValueError: Invalid file descriptor

    """
    cdef int n
    try:
        n = f.fileno()
    except AttributeError:
        n = f
    if n < 0 or n >= FD_SETSIZE:
        raise ValueError("Invalid file descriptor")
    return n


cdef class PSelecter:
    """
    This class gives an interface to the ``pselect`` system call.

    It can be used in a ``with`` statement to block given signals
    such that they can only occur during the :meth:`pselect()` or
    :meth:`sleep()` calls.

    As an example, we block the ``SIGHUP`` and ``SIGALRM`` signals and
    then raise a ``SIGALRM`` signal. The interrupt will only be seen
    during the :meth:`sleep` call::

        >>> from cysignals import AlarmInterrupt
        >>> from cysignals.pselect import PSelecter
        >>> import os, signal, time
        >>> with PSelecter([signal.SIGHUP, signal.SIGALRM]) as sel:
        ...     os.kill(os.getpid(), signal.SIGALRM)
        ...     time.sleep(0.5)  # Simply sleep, no interrupt detected
        ...     try:
        ...         _ = sel.sleep(1)  # Interrupt seen here
        ...     except AlarmInterrupt:
        ...         print("Interrupt OK")
        Interrupt OK

    .. WARNING::

        If ``SIGCHLD`` is blocked inside the ``with`` block, then you
        should not use ``Popen().wait()`` or ``Process().join()``
        because those might block, even if the process has actually
        exited. Use non-blocking alternatives such as ``Popen.poll()``
        or ``multiprocessing.active_children()`` instead.
    """
    cdef sigset_t oldset
    cdef sigset_t blockset

    def __cinit__(self):
        """
        Store old signal mask, needed if this class is used *without*
        a ``with`` statement.

        EXAMPLES::

            >>> from cysignals.pselect import PSelecter
            >>> PSelecter()
            <cysignals.pselect.PSelecter ...>

        """
        cdef sigset_t emptyset
        sigemptyset(&emptyset)
        sigprocmask(SIG_BLOCK, &emptyset, &self.oldset)

    def __init__(self, block=[]):
        """
        Store list of signals to block during ``pselect()``.

        EXAMPLES::

            >>> from cysignals.pselect import PSelecter
            >>> from signal import *
            >>> PSelecter([SIGINT, SIGSEGV])
            <cysignals.pselect.PSelecter ...>

        """
        sigemptyset(&self.blockset)
        for sig in block:
            sigaddset(&self.blockset, sig)

    def __enter__(self):
        """
        Block signals chosen during :meth:`__init__` in this ``with`` block.

        OUTPUT: ``self``

        TESTS:

        Test nesting, where the inner ``with`` statements should have no
        influence, in particular they should not unblock signals which
        were already blocked upon entering::

            >>> from cysignals import AlarmInterrupt
            >>> from cysignals.pselect import PSelecter
            >>> import os, signal
            >>> with PSelecter([signal.SIGALRM]) as sel:
            ...     os.kill(os.getpid(), signal.SIGALRM)
            ...     with PSelecter([signal.SIGFPE]) as sel2:
            ...         _ = sel2.sleep(0.1)
            ...     with PSelecter([signal.SIGALRM]) as sel3:
            ...         _ = sel3.sleep(0.1)
            ...     try:
            ...         _ = sel.sleep(0.1)
            ...     except AlarmInterrupt:
            ...         print("Interrupt OK")
            Interrupt OK

        """
        sigprocmask(SIG_BLOCK, &self.blockset, &self.oldset)
        return self

    def __exit__(self, type, value, traceback):
        """
        Reset signal mask to what it was before :meth:`__enter__`.

        EXAMPLES:

        Install a ``SIGCHLD`` handler::

            >>> import signal
            >>> def child_handler(sig, frame):
            ...     global got_child
            ...     got_child = 1
            >>> _ = signal.signal(signal.SIGCHLD, child_handler)
            >>> got_child = 0

        Start a process which will cause a ``SIGCHLD`` signal::

            >>> import time
            >>> from multiprocessing import *
            >>> from cysignals.pselect import PSelecter, interruptible_sleep
            >>> w = PSelecter([signal.SIGCHLD])
            >>> with w:
            ...     p = Process(target=time.sleep, args=(0.25,))
            ...     t0 = time.time()
            ...     p.start()

        This ``sleep`` should be interruptible now::

            >>> interruptible_sleep(1)
            >>> t = time.time() - t0
            >>> (0.2 <= t <= 0.9) or t
            True
            >>> got_child
            1
            >>> p.join()

        """
        sigprocmask(SIG_SETMASK, &self.oldset, NULL)

    def pselect(self, rlist=[], wlist=[], xlist=[], timeout=None):
        """
        Wait until one of the given files is ready, or a signal has
        been received, or until ``timeout`` seconds have past.

        INPUT:

        - ``rlist`` -- (default: ``[]``) a list of files to wait for
          reading.

        - ``wlist`` -- (default: ``[]``) a list of files to wait for
          writing.

        - ``xlist`` -- (default: ``[]``) a list of files to wait for
          exceptions.

        - ``timeout`` -- (default: ``None``) a timeout in seconds,
          where ``None`` stands for no timeout.

        OUTPUT: A 4-tuple ``(rready, wready, xready, tmout)`` where the
        first three are lists of file descriptors which are ready,
        that is a subset of ``(rlist, wlist, xlist)``. The fourth is a
        boolean which is ``True`` if and only if the command timed out.
        If ``pselect`` was interrupted by a signal, the output is
        ``([], [], [], False)``.

        .. SEEALSO::

            Use the :meth:`sleep` method instead if you don't care about
            file descriptors.

        EXAMPLES:

        The file ``/dev/null`` should always be available for reading
        and writing::

            >>> from cysignals.pselect import PSelecter
            >>> f = open(os.devnull, "r+")
            >>> sel = PSelecter()
            >>> sel.pselect(rlist=[f])
            ([<...'/dev/null'...>], [], [], False)
            >>> sel.pselect(wlist=[f])
            ([], [<...'/dev/null'...>], [], False)

        A list of various files, all of them should be ready for
        reading. Also create a pipe, which should be ready for
        writing, but not reading (since nothing has been written)::

            >>> import os, sys
            >>> f = open(os.devnull, "r")
            >>> g = open(sys.executable, "r")
            >>> (pr, pw) = os.pipe()
            >>> r, w, x, t = PSelecter().pselect([f,g,pr,pw], [pw], [pr,pw])
            >>> len(r), len(w), len(x), t
            (2, 1, 0, False)

        Checking for exceptions on the pipe should simply time out::

            >>> sel.pselect(xlist=[pr,pw], timeout=0.2)
            ([], [], [], True)

        TESTS:

        It is legal (but silly) to list the same file multiple times::

            >>> r, w, x, t = PSelecter().pselect([f,g,f,f,g])
            >>> len(r)
            5

        Invalid input::

            >>> PSelecter().pselect([None])
            Traceback (most recent call last):
            ...
            TypeError: an integer is required

        Open a file and close it, but save the (invalid) file
        descriptor::

            >>> f = open(os.devnull, "r")
            >>> n = f.fileno()
            >>> f.close()
            >>> PSelecter().pselect([n])
            Traceback (most recent call last):
            ...
            OSError: ...

        """
        # Convert given lists to fd_set
        cdef fd_set rfds, wfds, xfds
        FD_ZERO(&rfds)
        FD_ZERO(&wfds)
        FD_ZERO(&xfds)

        cdef int nfds = 0
        cdef int n
        for f in rlist:
            n = get_fileno(f)
            if (n >= nfds): nfds = n + 1
            FD_SET(n, &rfds)
        for f in wlist:
            n = get_fileno(f)
            if (n >= nfds): nfds = n + 1
            FD_SET(n, &wfds)
        for f in xlist:
            n = get_fileno(f)
            if (n >= nfds): nfds = n + 1
            FD_SET(n, &xfds)

        cdef double tm
        cdef timespec tv
        cdef timespec *ptv = NULL
        cdef int ret
        if timeout is not None:
            tm = timeout
            if tm < 0:
                tm = 0
            tv.tv_sec = <long>tm
            tv.tv_nsec = <long>(1e9 * (tm - <double>tv.tv_sec))
            ptv = &tv

        ret = pselect(nfds, &rfds, &wfds, &xfds, ptv, &self.oldset)
        # No file descriptors ready => timeout
        if ret == 0:
            return ([], [], [], True)

        # Error?
        if ret < 0:
            if libc.errno.errno == libc.errno.EINTR:
                return ([], [], [], False)
            PyErr_SetFromErrno(OSError)

        # Figure out which file descriptors to return
        rready = []
        wready = []
        xready = []
        for f in rlist:
            n = get_fileno(f)
            if FD_ISSET(n, &rfds):
                rready.append(f)
        for f in wlist:
            n = get_fileno(f)
            if FD_ISSET(n, &wfds):
                wready.append(f)
        for f in xlist:
            n = get_fileno(f)
            if FD_ISSET(n, &xfds):
                xready.append(f)

        return (rready, wready, xready, False)

    def sleep(self, timeout=None):
        """
        Wait until a signal has been received, or until ``timeout``
        seconds have past.

        This is implemented as a special case of :meth:`pselect` with
        empty lists of file descriptors.

        INPUT:

        - ``timeout`` -- (default: ``None``) a timeout in seconds,
          where ``None`` stands for no timeout.

        OUTPUT: A boolean which is ``True`` if the call timed out,
        False if it was interrupted.

        EXAMPLES:

        A simple wait with timeout::

            >>> from cysignals.pselect import PSelecter
            >>> sel = PSelecter()
            >>> sel.sleep(timeout=0.1)
            True

        0 or negative time-outs are allowed, ``sleep`` should then
        return immediately::

            >>> sel.sleep(timeout=0)
            True
            >>> sel.sleep(timeout=-123.45)
            True

        """
        return self.pselect(timeout=timeout)[3]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup
from distutils.command.build import build as _build
from distutils.command.build_py import build_py as _build_py
from setuptools.command.install import install as _install
from setuptools.command.bdist_egg import bdist_egg as _bdist_egg
from setuptools.extension import Extension
from Cython.Build.Dependencies import cythonize

import warnings
warnings.simplefilter("always")

import os
from glob import glob

opj = os.path.join


cythonize_dir = "build"

kwds = dict(include_dirs=[opj("src", "cysignals"),
                          opj(cythonize_dir, "src"),
                          opj(cythonize_dir, "src", "cysignals")],
            depends=glob(opj("src", "cysignals", "*.h")))

extensions = [
    Extension("cysignals.signals", ["src/cysignals/signals.pyx"], **kwds),
    Extension("cysignals.alarm", ["src/cysignals/alarm.pyx"], **kwds),
    Extension("cysignals.pselect", ["src/cysignals/pselect.pyx"], **kwds),
    Extension("cysignals.tests", ["src/cysignals/tests.pyx"], **kwds),
]


def write_if_changed(filename, text):
    """
    Write ``text`` to ``filename`` but only if it differs from the
    current content of ``filename``. If needed, the file and the
    containing directory are created.
    """
    try:
        f = open(filename, "r+")
    except IOError:
        try:
            os.makedirs(os.path.dirname(filename))
        except OSError:
            pass
        f = open(filename, "w")
    else:
        if f.read() == text:
            # File is up-to-date
            f.close()
            return
        f.seek(0)
        f.truncate()

    print("generating {0}".format(filename))
    f.write(text)
    f.close()


# Run Distutils
class build(_build):
    def run(self):
        """
        Run ``./configure`` and Cython first.
        """
        config_h = opj(cythonize_dir, "src", "config.h")
        if not os.path.isfile(config_h):
            import subprocess
            subprocess.check_call(["make", "configure"])
            subprocess.check_call(["sh", "configure"])

        self.create_init_pxd()

        dist = self.distribution
        ext_modules = dist.ext_modules
        if ext_modules:
            dist.ext_modules[:] = self.cythonize(ext_modules)

        _build.run(self)

    def cythonize(self, extensions):
        return cythonize(extensions,
                build_dir=cythonize_dir, include_path=["src", os.path.join(cythonize_dir, "src")])

    def create_init_pxd(self):
        """
        Create an ``__init__.pxd`` file in the build directory. This
        file will then be installed.

        The ``__init__.pxd`` file sets the correct compiler options for
        packages using cysignals.
        """
        # Determine installation directory
        inst = self.get_finalized_command("install")
        try:
            platlib = inst.noroot_platlib
        except AttributeError:
            platlib = inst.install_platlib
        install_dir = opj(platlib, "cysignals")

        # The variable "init_pxd" is the string which should be written to
        # __init__.pxd
        init_pxd = "# distutils: include_dirs = {0}\n".format(install_dir)
        # Append __init__.pxd from configure
        init_pxd += self.get_init_pxd()

        init_pxd_file = opj(self.build_lib, "cysignals", "__init__.pxd")
        write_if_changed(init_pxd_file, init_pxd)

    def get_init_pxd(self):
        """
        Get the contents of ``__init__.pxd`` as generated by configure.
        """
        configure_init_pxd_file = opj(cythonize_dir, "src", "cysignals", "__init__.pxd")
        with open(configure_init_pxd_file, "r") as f:
            return f.read()


class build_py(_build_py):
    """
    Custom distutils build_py class. For every package FOO, we also
    check package data for a "fake" FOO-cython package.
    """
    def get_data_files(self):
        """Generate list of '(package,src_dir,build_dir,filenames)' tuples"""
        data = []
        if not self.packages:
            return data
        for package in self.packages:
            for src_package in [package, package + "-cython"]:
                # Locate package source directory
                src_dir = self.get_package_dir(src_package)

                # Compute package build directory
                build_dir = os.path.join(*([self.build_lib] + package.split('.')))

                # Length of path to strip from found files
                plen = 0
                if src_dir:
                    plen = len(src_dir)+1

                # Strip directory from globbed filenames
                filenames = [
                    file[plen:] for file in self.find_data_files(src_package, src_dir)
                    ]
                data.append((package, src_dir, build_dir, filenames))
        return data


class install(_install):
    # When handling the --root option, keep the original paths prefixed
    # with "noroot" instead of "install"
    def change_roots(self, *names):
        for name in names:
            path = getattr(self, "install_" + name)
            setattr(self, "noroot_" + name, path)
        _install.change_roots(self, *names)


class no_egg(_bdist_egg):
    def run(self):
        from distutils.errors import DistutilsOptionError
        raise DistutilsOptionError("The package cysignals will not function correctly when built as egg. Therefore, it cannot be installed using 'python setup.py install' or 'easy_install'. Instead, use 'pip install' to install cysignals.")


setup(
    name="cysignals",
    author=u"Martin R. Albrecht, François Bissey, Volker Braun, Jeroen Demeyer",
    author_email="sage-devel@googlegroups.com",
    version=open("VERSION").read().strip(),
    url="https://github.com/sagemath/cysignals",
    license="GNU Lesser General Public License, version 3 or later",
    description="Interrupt and signal handling for Cython",
    long_description=open('README.rst').read(),
    platforms=["POSIX"],

    ext_modules=extensions,
    packages=["cysignals"],
    package_dir={"cysignals": opj("src", "cysignals"),
                 "cysignals-cython": opj(cythonize_dir, "src", "cysignals")},
    package_data={"cysignals": ["*.pxi", "*.pxd", "*.h"],
                  "cysignals-cython": ["*.h"]},
    scripts=glob(opj("src", "scripts", "*")),
    cmdclass=dict(build=build, build_py=build_py, install=install, bdist_egg=no_egg)
)



# Run by ``cysignals-CSI`` in gdb's Python interpreter.
#*****************************************************************************
#       Copyright (C) 2013 Volker Braun <vbraun.name@gmail.com>
#
#  cysignals is free software: you can redistribute it and/or modify it
#  under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  cysignals is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with cysignals.  If not, see <http://www.gnu.org/licenses/>.
#
#*****************************************************************************

import os
import sys
import glob

import gdb
from Cython.Debugger import libpython, libcython
from Cython.Debugger.libcython import cy, CythonCommand

try:
    if not color:
        libcython.pygments = None  # disable escape-sequence coloring
except (NameError, AttributeError):
    pass


def cython_debug_files():
    """
    Cython extra debug information files
    """
    try:
        SAGE_SRC = os.environ['SAGE_SRC']
    except KeyError:
        return []
    pattern = os.path.join(SAGE_SRC, 'build', 'cython_debug',
                           'cython_debug_info_*')
    return glob.glob(pattern)

print('\n\n')
print('Cython backtrace')
print('----------------')

# The Python interpreter in GDB does not do automatic backtraces for you
try:
    for f in cython_debug_files():
        cy.import_.invoke(f, None)

    class Backtrace(CythonCommand):
        name = 'cy fullbt'
        alias = 'cy full_backtrace'
        command_class = gdb.COMMAND_STACK
        completer_class = gdb.COMPLETE_NONE
        cy = cy

        def print_stackframe(self, frame, index, is_c=False):
            if not is_c and self.is_python_function(frame):
                pyframe = libpython.Frame(frame).get_pyop()
                if pyframe is None or pyframe.is_optimized_out():
                    # print this python function as a C function
                    return self.print_stackframe(frame, index, is_c=True)
                func_name = pyframe.co_name
                func_cname = 'PyEval_EvalFrameEx'
                func_args = []
            elif self.is_cython_function(frame):
                cyfunc = self.get_cython_function(frame)
                f = lambda arg: self.cy.cy_cvalue.invoke(arg, frame=frame)  # noqa

                func_name = cyfunc.name
                func_cname = cyfunc.cname
                func_args = []  # [(arg, f(arg)) for arg in cyfunc.arguments]
            else:
                func_name = frame.name()
                func_cname = func_name
                func_args = []

            try:
                gdb_value = gdb.parse_and_eval(func_cname)
            except (RuntimeError, TypeError):
                func_address = 0
            else:
                func_address = int(str(gdb_value.address).split()[0], 0)

            source_desc, lineno = self.get_source_desc(frame)
            a = ', '.join('%s=%s' % (name, val) for name, val in func_args)
            out = '#%-2d 0x%016x in %s(%s)' % (index, func_address, func_name, a)
            if source_desc.filename is not None:
                out += 'at %s:%s' % (source_desc.filename, lineno)
            print(out)
            try:
                source = source_desc.get_source(lineno - 5, lineno + 5,
                                                mark_line=lineno, lex_entire=True)
                print(source)
            except gdb.GdbError:
                pass

        def invoke(self, args, from_tty):
            self.newest_first_order(args, from_tty)

        def newest_first_order(self, args, from_tty):
            frame = gdb.newest_frame()
            index = 0
            while frame:
                frame.select()
                self.print_stackframe(frame, index)
                index += 1
                frame = frame.older()

        def newest_last_order(self, args, from_tty):
            frame = gdb.newest_frame()
            n_frames = 0
            while frame.older():
                frame = frame.older()
                n_frames += 1
            index = 0
            while frame:
                frame.select()
                self.print_stackframe(frame, index)
                index += 1
                frame = frame.newer()

    trace = Backtrace.register()
    trace.invoke(None, None)


except Exception as e:
    import traceback
    traceback.print_exc()

sys.stdout.flush()



#!/usr/bin/env python
#
# Run doctests for cysignals
#
# We add the ELLIPSIS flag by default and we run all tests even if
# one fails.
#

import doctest
import sys

flags = doctest.ELLIPSIS

filenames = sys.argv[1:]
print("Doctesting {} files.".format(len(filenames)))

success = True
for f in filenames:
    print(f)
    failures, _ = doctest.testfile(f, module_relative=False, optionflags=flags)
    if failures:
        success = False

sys.exit(0 if success else 1)



# distutils: language = c++

from cysignals.signals cimport sig_check
from cysignals.memory cimport check_allocarray

from libc.math cimport sin

def sine_sum(double x, long count):
    cdef double s = 0
    cdef long i
    for i in range(count):
        sig_check()
        s += sin(i*x)
    return s

cdef long* safe_range_long(long count) except? NULL:
    """
    This function can be safely called within a sig_on block.

    With an ordinary malloc, this is not the case since the internal
    state of the heap would be messed up if an interrupt happens during
    malloc().
    """
    cdef long* a = <long*>check_allocarray(count, sizeof(long))
    for i in range(count):
        a[i] = i
    return a
